package org.netkernel.layer0.meta.impl;

import org.netkernel.layer0.bnf.BNFGrammarFactory;
import org.netkernel.layer0.bnf.IGrammarGroup;
import org.netkernel.layer0.meta.IEndpointArgumentMeta;
import org.netkernel.layer0.urii.ActiveURIEncoding;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;

public class ActiveEndpointMetaBuilder extends EndpointMetaBuilder
{
	public static final String ARG_ACTIVE_TYPE=ParsedIdentifierImpl.ARG_ACTIVE_TYPE;
	
	private IGrammarGroup mArgumentInterleave;
	
	public ActiveEndpointMetaBuilder(String aActiveType)
	{	IGrammarGroup grammarRoot = BNFGrammarFactory.createGroup(null);
		BNFGrammarFactory.createText(grammarRoot, "active:");
		IGrammarGroup type=BNFGrammarFactory.createGroup(grammarRoot, ARG_ACTIVE_TYPE,null);
		BNFGrammarFactory.createText(type, aActiveType);
		mArgumentInterleave=BNFGrammarFactory.createInterleave(grammarRoot);
		setGrammarRoot(grammarRoot);
	}
	
	public ActiveEndpointMetaBuilder(String[] aActiveTypes)
	{	IGrammarGroup grammarRoot = BNFGrammarFactory.createGroup(null);
		BNFGrammarFactory.createText(grammarRoot, "active:");
		IGrammarGroup type = BNFGrammarFactory.createChoiceGroup(grammarRoot, ARG_ACTIVE_TYPE,null);
		for (int i=0; i<aActiveTypes.length; i++)
		{	BNFGrammarFactory.createText(type, aActiveTypes[i]);
		}
		mArgumentInterleave=BNFGrammarFactory.createInterleave(grammarRoot);
		setGrammarRoot(grammarRoot);
	}
	
	public void declareArgument(String aName, String aDescription, boolean aMandatory, String aDefaultValue)
	{	declareArgument(aName,aDescription,aMandatory,aDefaultValue,null);
	}
	
	public void declareArgument(String aName, String aDescription, boolean aMandatory, String aDefaultValue, Class aRepresentationClass)
	{
		Class[] representations;
		if (aRepresentationClass!=null)
		{	representations = new Class[]{aRepresentationClass};
		}
		else
		{	representations=null;
		}
		IEndpointArgumentMeta arg = new EndpointArgumentMetaImpl(aName,aDescription,null,null,aDefaultValue,true,representations);
		addArgument(arg);
		
		createArgument(mArgumentInterleave, aName, aMandatory?1:0,1);		
	}
	
	public void declareAdditionalUndefinedArguments()
	{	createVarargs(mArgumentInterleave);
	}
		
	public static IGrammarGroup createVarargs(IGrammarGroup aParent)
	{	IGrammarGroup g1 = BNFGrammarFactory.createGroup(aParent,0,Integer.MAX_VALUE);
		BNFGrammarFactory.createText(g1, "+");
		IGrammarGroup g2 = BNFGrammarFactory.createGroup(g1,BNFGrammarFactory.ARG_UNDEFINED_NAME,null);
		BNFGrammarFactory.createRegex(g2, BNFGrammarFactory.BUILTIN_NMTOKEN);
		BNFGrammarFactory.createText(g1, "@");
		g2 = BNFGrammarFactory.createGroup(g1,BNFGrammarFactory.ARG_UNDEFINED_VALUE, ActiveURIEncoding.INSTANCE);
		BNFGrammarFactory.createRegex(g2, BNFGrammarFactory.BUILTIN_ACTIVE_ESCAPED_URI_SIMPLE);
		return g1;
	}
	
	public static IGrammarGroup createArgument(IGrammarGroup aParent, String aName, int aMin, int aMax)
	{	IGrammarGroup g0 = BNFGrammarFactory.createGroup(aParent, aMin, aMax);
		BNFGrammarFactory.createText(g0, "+"+aName+"@");
		IGrammarGroup g1 = BNFGrammarFactory.createGroup(g0, aName, ActiveURIEncoding.INSTANCE);
		BNFGrammarFactory.createRegex(g1, BNFGrammarFactory.BUILTIN_ACTIVE_ESCAPED_URI_SIMPLE);
		return g0;
	}
	
}
