/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf;

import org.netkernel.layer0.meta.IPrototypeParameterMeta;

/**
 * High-level helper methods for implementing accessors. This Interface provides methods that will cover 80% 
 * of Accessor development cases.  For additional convenience methods to source/sink resources see <a href="INKFRequestContext.html">INKFRequestContext</a>
 * @author  tab
 */
public interface INKFBasicContext
{
	//*************************
	// this request methods
	//*************************
	
	/** Return a wrapper around the request which initiated
	 * the invocation of this accessor
	 * @exception IllegalStateException thrown if no parent request initiated this invocation
*/
	INKFRequestReadOnly getThisRequest() throws IllegalStateException;
	
	//*************************
	// subrequest methods
	//*************************
	
	/** Create a request with a given base URI. Equivalent
	 to calling createSubRequest().setIdentifier(aURI)
	 * @param aIdentifier
	 * @exception NKFException thrown if aIdentifier is relative and cwu not inherited from parent
	 */
    INKFRequest createRequest(String aIdentifier) throws NKFException;
	
    /** Create a request which is targeted at a specific endpoint. 
     * @param aEndpointId the identifier of an endpoint that can be resolved within the space. This endpoint
     * must expose interface metadata to be successfully called in this way
     * @throws NKFException thrown if endpoint cannot be resolved or doesn't expose necessary metadata
     */
    INKFRequest createRequestToEndpoint(String aEndpointId) throws NKFException;
    
	/** Issue a request.  Make a secondary request to the kernel for scheduling.
	 *  The request is synchronous.
	 * @param aRequest
	 * @return the resulting resource representation
	 * @exception NKFException if there is a failure executing the request
	 */
	Object issueRequest(INKFRequest aRequest) throws NKFException;
	
	/** Issue the request returning back the whole response so
	 *  that metadata can be interrogated.
	 * @param aRequest
	 * @return the response object
	 * @exception NKFException if there is a failure executing the request
	 */
	INKFResponseReadOnly issueRequestForResponse(INKFRequest aRequest) throws NKFException;

	/** Issue an asynchronous request. Make a secondary request to the kernel
	 * for scheduling
	 *@param aRequest
	 *@return 
	 */
	INKFAsyncRequestHandle issueAsyncRequest(INKFRequest aRequest) throws NKFException;
	
	/** Sets the CWU - the identifier used to resolve relative identifiers against
	 * @param aIdentifier
	 */
	void setCWU(String aIdentifier);
	
	/** Returns the current CWU
=	 */
	String getCWU();
	
	/** Return the identifier of the language runtimes script if any. Useful to set the CWU
	 */
	String getRuntimeSourceIdentifier();
	
	//*************************
	// response methods
	//*************************
	
	/** Create a response from this endpoint based on this representation of the resource.
	 *  The returned INKFResponse object can then be further configured. 
	 *  createResponseFrom(null) differs from not setting any response because by default setting
	 *  no response will return null but it will be expired because the endpoint will be assumed to have
	 *  a side effect.
	 * @param aRepresentation a representation to be used as the basis for the response
	 */
	INKFResponse createResponseFrom(Object aRepresentation);
	
	/** Create a response from this endpoint based on this response from a subrequest.
	 *  Metadata will be propagated.
	 * @param aResponse an response from a subrequest which should be used as the basis for
	 * the response. 
	 */
	INKFResponse createResponseFrom(INKFResponseReadOnly aResponse);
	
	/** For asynchronous endpoints that don't want to return a response immediately
	 *  they must call this method to ensure that no synchronous response is
	 *  returned to the requestee when they complete.
	 */
	void setNoResponse();
	
	/** Returns true if a response has been set
	 */
	boolean isResponseSet();
	
	/** Unset the response of this endpoint.  This restores the response back to the initial and default
	 *  state - a null response.
	 */
	void unsetResponse();
	
	/** Declare additional latebound parameters (useful for language runtimes)
	 * @param aParameterDefn an array of new parameters, these will be added to existing ones
	 * @throws NKFException thrown if there is a problem
	 */
	void declareParameters(IPrototypeParameterMeta[] aParameterDefn) throws NKFException;	
	
	/** <b>Advanced</b> Return the kernel Context API for a set of low-level
	 * APIs for advanced use.
	 */
	INKFKernelContext getKernelContext();
	
	
	
}