/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf;

import java.io.IOException;
import java.io.Writer;

import org.netkernel.layer0.util.PairList;
import org.netkernel.urii.INetKernelException;
import org.netkernel.urii.impl.NetKernelException;
import org.netkernel.util.XMLUtils;

/**
 * Wrapper around NetkernelException, might be thrown when issuing
 * subrequests from interfaces in this package
 * @author  tab
 */
public class NKFException extends NetKernelException
{
	private String mLocation;
	private boolean mShowNativeStack;
	private PairList mAdditionFields=null;
	
	/** Creates a new instance of NKFException */
	public NKFException(String aId)
	{	super(aId);
	}
	
	/** Creates a new instance of NKFException */
	public NKFException(String aId, String aMessage)
	{	super(aId,aMessage);
	}
	
	/** Creates a new instance of NKFException */
	public NKFException(String aId, String aMessage, Throwable aCause)
	{	super(aId,aMessage,aCause);
	}
	
	/** Creates a new instance of NKFException */
	public NKFException(String aId, String aMessage, String aLocation, Throwable aCause)
	{	super(aId,aMessage,aCause);
		mLocation=aLocation;
	}
	
	public NKFException(INetKernelException aKernelException)
	{	super(null,null,aKernelException);
	}
	
	public String getLocation()
	{	return mLocation;
	}
	
	public boolean showNativeStack()
	{	return mShowNativeStack;
	}
	
	public void setShowNativeStack()
	{	mShowNativeStack=true;
	}
	
	/** Add an additional field into the exception. Can be used for adding semantic state to the exception trace
	 * @param aName should be a safe to use as an XML element
	 * @param aValue
	 */
	public void addAdditionalField(String aName, String aValue)
	{	if (mAdditionFields==null)
		{	mAdditionFields=new PairList(4);
		}
		mAdditionFields.put(aName,aValue);
	}
	
	/* For internal use only. 
	 * @see org.netkernel.urii.impl.NetKernelException#writeAdditionalFieldsAsXML(java.io.Writer, int)
	 */
	public void writeAdditionalFieldsAsXML(Writer aWriter, int aIndent) throws IOException
	{	if (mLocation!=null)
		{	XMLUtils.writeElement(aWriter, "location",mLocation,aIndent);
		}
		if (mAdditionFields!=null)
		{	for (int i=0; i<mAdditionFields.size(); i++)
			{	String key=(String)mAdditionFields.getValue1(i);
				String value=(String)mAdditionFields.getValue2(i);
				XMLUtils.writeOpenTag(aWriter, "field", aIndent);
				XMLUtils.writeElement(aWriter, "name",key,aIndent+1);
				XMLUtils.writeElement(aWriter, "value",value,aIndent+1);
				XMLUtils.writeCloseTag(aWriter, "field", aIndent);
			}
		}
	}	
}